@@ -14,21 +14,21 @@ TODO:
 
 # This file is part of the SpatialMath toolbox for Python
 # https://github.com/petercorke/spatialmath-python
-# 
+#
 # MIT License
-# 
+#
 # Copyright (c) 1993-2020 Peter Corke
-# 
+#
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
 # in the Software without restriction, including without limitation the rights
 # to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 # copies of the Software, and to permit persons to whom the Software is
 # furnished to do so, subject to the following conditions:
-# 
+#
 # The above copyright notice and this permission notice shall be included in all
 # copies or substantial portions of the Software.
-# 
+#
 # THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 # IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 # FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
@@ -38,11 +38,12 @@ TODO:
 # SOFTWARE.
 
 # Contributors:
-# 
+#
 #     1. Luis Fernando Lara Tobar and Peter Corke, 2008
 #     2. Josh Carrigg Hodson, Aditya Dua, Chee Ho Chan, 2017 (robopy)
 #     3. Peter Corke, 2020
 
+# pylint: disable=invalid-name
 
 import sys
 import math
@@ -51,36 +52,32 @@ from spatialmath.base import argcheck
 from spatialmath.base import vectors as vec
 from spatialmath.base import transformsNd as trn
 from spatialmath.base import quaternions as quat
-
+from spatialmath.base import animate
 
 try:  # pragma: no cover
     # print('Using SymPy')
     import sympy as sym
 
-    def issymbol(x):
+    def _issymbol(x):
         return isinstance(x, sym.Symbol)
-except BaseException:
-    def issymbol(x):
+except ImportError:
+    def _issymbol(x):  # pylint: disable=unused-argument
         return False
 
 _eps = np.finfo(np.float64).eps
 
-
-def colvec(v):
-    return np.array(v).reshape((len(v), 1))
-
 # ---------------------------------------------------------------------------------------#
 
 
 def _cos(theta):
-    if issymbol(theta):
+    if _issymbol(theta):
         return sym.cos(theta)
     else:
         return math.cos(theta)
 
 
 def _sin(theta):
-    if issymbol(theta):
+    if _issymbol(theta):
         return sym.sin(theta)
     else:
         return math.sin(theta)
@@ -772,7 +769,7 @@ def tr2eul(T, unit='rad', flip=False, check=False):
 
 
 def tr2rpy(T, unit='rad', order='zyx', check=False):
-    """
+    r"""
     Convert SO(3) or SE(3) to roll-pitch-yaw angles
 
     :param R: SO(3) or SE(3) matrix
@@ -897,12 +894,16 @@ def tr2rpy(T, unit='rad', order='zyx', check=False):
 
 
 # ---------------------------------------------------------------------------------------#
-def trlog(T, check=True):
+def trlog(T, check=True, twist=False):
     """
     Logarithm of SO(3) or SE(3) matrix
 
     :param T: SO(3) or SE(3) matrix
     :type T: numpy.ndarray, shape=(3,3) or (4,4)
+    :param check: check that matrix is valid
+    :type check: bool
+    :param twist: return a twist vector instead of matrix [default]
+    :type twist: bool
     :return: logarithm
     :rtype: numpy.ndarray, shape=(3,3) or (4,4)
     :raises: ValueError
@@ -925,30 +926,39 @@ def trlog(T, check=True):
 
         if trn.iseye(T):
             # is identity matrix
-            return np.zeros((4, 4))
+            if twist:
+                return np.zeros((6,))
+            else:
+                return np.zeros((4, 4))
         else:
             [R, t] = trn.tr2rt(T)
 
             if trn.iseye(R):
                 # rotation matrix is identity
-                skw = np.zeros((3, 3))
-                v = t
-                theta = 1
+                if twist:
+                    return np.r_[t, 0, 0, 0]
+                else:
+                    return trn.rt2m(np.zeros((3, 3)), t)
             else:
                 S = trlog(R, check=False)  # recurse
                 w = trn.vex(S)
                 theta = vec.norm(w)
-                skw = trn.skew(w / theta)
-                Ginv = np.eye(3) / theta - skw / 2 + (1 / theta - 1 / np.tan(theta / 2) / 2) * skw @ skw
+                Ginv = np.eye(3) - S / 2 + (1 / theta - 1 / np.tan(theta / 2) / 2) / theta * S @ S
                 v = Ginv @ t
-            return trn.rt2m(skw, v) * theta
+                if twist:
+                    return np.r_[v, w]
+                else:
+                    return trn.rt2m(S, v)
 
     elif isrot(T, check=check):
         # deal with rotation matrix
         R = T
         if trn.iseye(R):
             # matrix is identity
-            return np.zeros((3, 3))
+            if twist:
+                return np.zeros((3,))
+            else:
+                return np.zeros((3, 3))
         elif abs(np.trace(R) + 1) < 100 * _eps:
             # check for trace = -1
             #   rotation by +/- pi, +/- 3pi etc.
@@ -959,19 +969,25 @@ def trlog(T, check=True):
             col = R[:, k] + I[:, k]
             w = col / np.sqrt(2 * (1 + mx))
             theta = math.pi
-            return trn.skew(w * theta)
+            if twist:
+                return w * theta
+            else:
+                return trn.skew(w * theta)
         else:
             # general case
             theta = np.arccos((np.trace(R) - 1) / 2)
             skw = (R - R.T) / 2 / np.sin(theta)
-            return skw * theta
+            if twist:
+                return trn.vex(skw * theta)
+            else:
+                return skw * theta
     else:
         raise ValueError("Expect SO(3) or SE(3) matrix")
 
 # ---------------------------------------------------------------------------------------#
 
 
-def trexp(S, theta=None):
+def trexp(S, theta=None, check=True):
     """
     Exponential of so(3) or se(3) matrix
 
@@ -1019,6 +1035,8 @@ def trexp(S, theta=None):
         # se(3) case
         if argcheck.ismatrix(S, (4, 4)):
             # augmentented skew matrix
+            if check:
+                assert trn.isskewa(S), 'argument must be a valid se(3) element'
             tw = trn.vexa(S)
         else:
             # 6 vector
@@ -1026,7 +1044,7 @@ def trexp(S, theta=None):
 
         if vec.iszerovec(tw):
             return np.eye(4)
-        
+
         if theta is None:
             (tw, theta) = vec.unittwist_norm(tw)
         else:
@@ -1038,7 +1056,7 @@ def trexp(S, theta=None):
         t = tw[0:3]
         w = tw[3:6]
 
-        R = trn._rodrigues(w, theta)
+        R = trn.rodrigues(w, theta)
 
         skw = trn.skew(w)
         V = np.eye(3) * theta + (1.0 - math.cos(theta)) * skw + (theta - math.sin(theta)) * skw @ skw
@@ -1049,6 +1067,8 @@ def trexp(S, theta=None):
         # so(3) case
         if argcheck.ismatrix(S, (3, 3)):
             # skew symmetric matrix
+            if check:
+                assert trn.isskew(S), 'argument must be a valid so(3) element'
             w = trn.vex(S)
         else:
             # 3 vector
@@ -1058,14 +1078,15 @@ def trexp(S, theta=None):
             assert vec.isunitvec(w), 'If theta is specified S must be a unit twist'
 
         # do Rodrigues' formula for rotation
-        return trn._rodrigues(w, theta)
+        return trn.rodrigues(w, theta)
     else:
         raise ValueError(" First argument must be SO(3), 3-vector, SE(3) or 6-vector")
 
+
 def trnorm(T):
     """
     Normalize an SO(3) or SE(3) matrix
-    
+
     :param T: SO(3) or SE(3) matrix
     :type T1: np.ndarray, shape=(3,3) or (4,4)
     :param T1: second SE(3) matrix
@@ -1077,112 +1098,135 @@ def trnorm(T):
       = [N,O,A] the O and A vectors are made unit length and the normal vector
       is formed from N = O x A, and then we ensure that O and A are orthogonal
       by O = A x N.
-    
+
     - ``trnorm(T)`` as above but the rotational submatrix of the homogeneous
       transformation T (4x4) is normalised while the translational part is
       unchanged.
 
     Notes:
-        
+
     - Only the direction of A (the z-axis) is unchanged.
-    - Used to prevent finite word length arithmetic causing transforms to 
+    - Used to prevent finite word length arithmetic causing transforms to
       become 'unnormalized'.
     """
 
     assert ishom(T) or isrot(T), 'expecting 3x3 or 4x4 hom xform'
-    
-    o = T[:3,1]
-    a = T[:3,2]
-    
+
+    o = T[:3, 1]
+    a = T[:3, 2]
+
     n = np.cross(o, a)        # N = O x A
     o = np.cross(a, n)        # (a)];
     R = np.stack((vec.unitvec(n), vec.unitvec(o), vec.unitvec(a)), axis=1)
-    
+
     if ishom(T):
-        return trn.rt2tr( R, T[:3,3] )
+        return trn.rt2tr(R, T[:3, 3])
     else:
         return R
-    
-def trinterp(end, start=None, s=None):
+
+
+def trinterp(start, end, s=None):
     """
     Interpolate SE(3) matrices
-    
-    :param end: final SE(3) matrix, value when s=1
-    :type end: np.ndarray, shape=(4,4)
-    :param start: initial SE(3) matrix, value when s=0, optional, defaults to identity
-    :type start: np.ndarray, shape=(4,4)
+
+    :param start: initial SO(3) or SE(3) matrix value when s=0, if None then identity is used
+    :type start: np.ndarray, shape=(3,3), (4,4)
+    :param end: final SO(3) or SE(3) matrix, value when s=1
+    :type end: np.ndarray, shape=(3,3), (4,4)
     :param s: interpolation coefficient, range 0 to 1
     :type s: float
-    :return: SE(3) matrix
-    :rtype: np.ndarray, shape=(4,4)
-    
-    - ``trinterp(T1, s=S)`` is a homogeneous transform (4x4) interpolated
-      between identity when S=0 and T1 when S=1.
-    - ``trinterp(T1, start=T0, s=S)`` as above but interpolated
-      between T0 when S=0 and T1 when S=1.  T0 and T1 are both homogeneous
-      transforms (4x4).
-    
+    :return: SO(3) or SE(3) matrix
+    :rtype: np.ndarray, shape=(3,3), (4,4)
+
+    - ``trinterp(None, T, S)`` is a homogeneous transform (4x4) interpolated
+      between identity when S=0 and T (4x4) when S=1.
+    - ``trinterp(T0, T1, S)`` as above but interpolated
+      between T0 (4x4) when S=0 and T1 (4x4) when S=1.
+    - ``trinterp(None, R, S)`` is a rotation matrix (3x3) interpolated
+      between identity when S=0 and R (3x3) when S=1.
+    - ``trinterp(R0, R1, S)`` as above but interpolated
+      between R0 (3x3) when S=0 and R1 (3x3) when S=1.
+
     Notes:
-        
+
     - Rotation is interpolated using quaternion spherical linear interpolation (slerp).
 
     :seealso: :func:`spatialmath.base.quaternions.slerp`, :func:`~spatialmath.base.transforms3d.trinterp2`
     """
 
     assert 0 <= s <= 1, 's outside interval [0,1]'
-    
-    if end is None:
-        #	TRINTERP(T, s)
-        
-        q0 = quat.r2q(trn.t2r(start))
-        p0 = transl(start)
-        
-        qr = quat.slerp(quat.eye(), q0, s)
-        pr = s * p0
+
+    if argcheck.ismatrix(end, (3, 3)):
+        # SO(3) case
+
+        if start is None:
+            #	TRINTERP(T, s)
+            q0 = quat.r2q(trn.t2r(end))
+            qr = quat.slerp(quat.eye(), q0, s)
+        else:
+            #	TRINTERP(T0, T1, s)
+            q0 = quat.r2q(trn.t2r(start))
+            q1 = quat.r2q(trn.t2r(end))
+            qr = quat.slerp(q0, q1, s)
+
+        return quat.q2r(qr)
+
+    elif argcheck.ismatrix(end, (4, 4)):
+        # SE(3) case
+        if start is None:
+            #	TRINTERP(T, s)
+            q0 = quat.r2q(trn.t2r(end))
+            p0 = transl(end)
+
+            qr = quat.slerp(quat.eye(), q0, s)
+            pr = s * p0
+        else:
+            #	TRINTERP(T0, T1, s)
+            q0 = quat.r2q(trn.t2r(start))
+            q1 = quat.r2q(trn.t2r(end))
+
+            p0 = transl(start)
+            p1 = transl(end)
+
+            qr = quat.slerp(q0, q1, s)
+            pr = p0 * (1 - s) + s * p1
+
+        return trn.rt2tr(quat.q2r(qr), pr)
     else:
-        #	TRINTERP(T0, T1, s)
-    
-        q0 = quat.r2q(trn.t2r(start))
-        q1 = quat.r2q(trn.t2r(end))
-        
-        p0 = transl(start)
-        p1 = transl(end)
-        
-        qr = quat.slerp(q0, q1, s)
-        pr = p0 * (1 - s) + s * p1;
-        
-    return trn.rt2tr(quat.q2r(qr), pr)
+        return ValueError('Argument must be SO(3) or SE(3)')
+
 
 def delta2tr(d):
     r"""
     Convert differential motion to SE(3)
-    
+
     :param d: differential motion as a 6-vector
     :type d: array_like
     :return: SE(3) matrix
     :rtype: np.ndarray, shape=(4,4)
 
-    ``T = delta2tr(d)`` is an SE(3) matrix representing differential 
+    ``T = delta2tr(d)`` is an SE(3) matrix representing differential
     motion :math:`d = [\delta_x, \delta_y, \delta_z, \theta_x, \theta_y, \theta_z`.
 
     Reference: Robotics, Vision & Control: Second Edition, P. Corke, Springer 2016; p67.
-    
+
     :seealso: :func:`~tr2delta`
     """
 
-    return np.eye(4,4) + trn.skewa(d)
-    
+    return np.eye(4, 4) + trn.skewa(d)
+
+
 def trinv(T):
     r"""
     Invert an SE(3) matrix
-    
+
     :param T: an SE(3) matrix
     :type T: np.ndarray, shape=(4,4)
     :return: SE(3) matrix
     :rtype: np.ndarray, shape=(4,4)
 
     Computes an efficient inverse of an SE(3) matrix:
-    
+
     :math:`\begin{pmatrix} {\bf R} & t \\ 0\,0\,0 & 1 \end{pmatrix}^{-1} =  \begin{pmatrix} {\bf R}^T & -{\bf R}^T t \\ 0\,0\, 0 & 1 \end{pmatrix}`
 
     """
@@ -1190,10 +1234,11 @@ def trinv(T):
     (R, t) = trn.tr2rt(T)
     return trn.rt2tr(R.T, -R.T@t)
 
+
 def tr2delta(T0, T1=None):
-    """
+    r"""
     Difference of SE(3) matrices as differential motion
-    
+
     :param T0: first SE(3) matrix
     :type T0: np.ndarray, shape=(4,4)
     :param T1: second SE(3) matrix
@@ -1202,22 +1247,22 @@ def tr2delta(T0, T1=None):
     :rtype: np.ndarray, shape=(6,)
 
 
-    - ``tr2delta(T0, T1)`` is the differential motion (6x1) corresponding to 
+    - ``tr2delta(T0, T1)`` is the differential motion (6x1) corresponding to
       infinitessimal motion (in the T0 frame) from pose T0 to T1 which are SE(3) matrices.
 
     - ``tr2delta(T)`` as above but the motion is from the world frame to the pose represented by T.
 
     The vector :math:`d = [\delta_x, \delta_y, \delta_z, \theta_x, \theta_y, \theta_z`
-    represents infinitessimal translation and rotation, and is an approximation to the 
+    represents infinitessimal translation and rotation, and is an approximation to the
     instantaneous spatial velocity multiplied by time step.
 
     Notes:
-        
+
     - D is only an approximation to the motion T, and assumes
       that T0 ~ T1 or T ~ eye(4,4).
     - Can be considered as an approximation to the effect of spatial velocity over a
       a time interval, average spatial velocity multiplied by time.
-    
+
     Reference: Robotics, Vision & Control: Second Edition, P. Corke, Springer 2016; p67.
 
     :seealso: :func:`~delta2tr`
@@ -1225,28 +1270,29 @@ def tr2delta(T0, T1=None):
 
     if T1 is None:
         # tr2delta(T)
-        
+
         assert ishom(T0), 'expecting SE(3) matrix'
         Td = T0
-        
+
     else:
         #  incremental transformation from T0 to T1 in the T0 frame
         Td = trinv(T0) @ T1
-    
+
     return np.r_[transl(Td), trn.vex(trn.t2r(Td) - np.eye(3))]
 
+
 def tr2jac(T, samebody=False):
     """
     SE(3) adjoint
-    
+
     :param T: an SE(3) matrix
     :type T: np.ndarray, shape=(4,4)
     :return: adjoint matrix
     :rtype: np.ndarray, shape=(6,6)
-    
+
     Computes an adjoint matrix that maps spatial velocity between two frames defined by
     an SE(3) matrix.  It acts like a Jacobian matrix.
-    
+
     - ``tr2jac(T)`` is a Jacobian matrix (6x6) that maps spatial velocity or
       differential motion from frame {A} to frame {B} where the pose of {B}
       relative to {A} is represented by the homogeneous transform T = :math:`{}^A {\bf T}_B`.
@@ -1256,13 +1302,13 @@ def tr2jac(T, samebody=False):
     """
 
     assert ishom(T), 'expecting an SE(3) matrix'
-    Z = np.zeros((3,3))
-    	
+    Z = np.zeros((3, 3))
+
     if samebody:
-        (R,t) = trn.tr2rt(T)
+        (R, t) = trn.tr2rt(T)
         return np.block([[R.T, (trn.skew(t)@R).T], [Z, R.T]])
     else:
-        R = trn.t2r(T);
+        R = trn.t2r(T)
         return np.block([[R.T, Z], [Z, R.T]])
 
 
@@ -1352,7 +1398,6 @@ def trprint(T, orient='rpy/zyx', label=None, file=sys.stdout, fmt='{:8.2g}', uni
         s += ' eul = {} {}'.format(_vec2s(fmt, angles), unit)
 
     elif a[0] == 'angvec':
-        pass
         # as a vector and angle
         (theta, v) = tr2angvec(T, unit)
         if theta == 0:
@@ -1375,182 +1420,184 @@ def _vec2s(fmt, v):
 
 try:
     import matplotlib.pyplot as plt
-    from mpl_toolkits.mplot3d import Axes3D
     _matplotlib_exists = True
-    
-except BaseException:  # pragma: no cover
-    def trplot(*args, **kwargs):
-        print('matplotlib is not installed: pip install matplotlib')
+except ImportError:  # pragma: no cover      
     _matplotlib_exists = False
-        
-if _matplotlib_exists:
-    def trplot(T, axes=None, block=True, dims=None, color='blue', frame=None, textcolor=None, labels=['X', 'Y', 'Z'], length=1, arrow=True, projection='ortho', rviz=False, wtl=0.2, width=1, d1=0.05, d2=1.15, **kwargs):
-        """
-        Plot a 3D coordinate frame
 
-        :param T: an SO(3) or SE(3) pose to be displayed as coordinate frame
-        :type: numpy.ndarray, shape=(3,3) or (4,4)
-        :param axes: the axes to plot into, defaults to current axes
-        :type axes: Axes3D reference
-        :param block: run the GUI main loop until all windows are closed, default True
-        :type block: bool
-        :param dims: dimension of plot volume as [xmin, xmax, ymin, ymax,zmin, zmax].
-                     If dims is [min, max] those limits are applied to the x-, y- and z-axes.
-        :type dims: array_like
-        :param color: color of the lines defining the frame
-        :type color: str
-        :param textcolor: color of text labels for the frame, default color of lines above
-        :type textcolor: str
-        :param frame: label the frame, name is shown below the frame and as subscripts on the frame axis labels
-        :type frame: str
-        :param labels: labels for the axes, defaults to X, Y and Z
-        :type labels: 3-tuple of strings
-        :param length: length of coordinate frame axes, default 1
-        :type length: float
-        :param arrow: show arrow heads, default True
-        :type arrow: bool
-        :param wtl: width-to-length ratio for arrows, default 0.2
-        :type wtl: float
-        :param rviz: show Rviz style arrows, default False
-        :type rviz: bool
-        :param projection: 3D projection: ortho [default] or persp
-        :type projection: str
-        :param width: width of lines, default 1
-        :type width: float
-        :param d1: distance of frame axis label text from origin, default 1.15
-        :type d2: distance of frame label text from origin, default 0.05
-        :return: axes containing the frame
-        :rtype: Axes3DSubplot
+def trplot(T, axes=None, block=True, dims=None, color='blue', frame=None,   # pylint: disable=unused-argument,function-redefined
+           textcolor=None, labels=('X', 'Y', 'Z'), length=1, arrow=True,
+           projection='ortho', rviz=False, wtl=0.2, width=1, d1=0.05,
+           d2=1.15, **kwargs):
+    """
+    Plot a 3D coordinate frame
 
-        Adds a 3D coordinate frame represented by the SO(3) or SE(3) matrix to the current axes.
+    :param T: an SO(3) or SE(3) pose to be displayed as coordinate frame
+    :type: numpy.ndarray, shape=(3,3) or (4,4)
+    :param axes: the axes to plot into, defaults to current axes
+    :type axes: Axes3D reference
+    :param block: run the GUI main loop until all windows are closed, default True
+    :type block: bool
+    :param dims: dimension of plot volume as [xmin, xmax, ymin, ymax,zmin, zmax].
+                    If dims is [min, max] those limits are applied to the x-, y- and z-axes.
+    :type dims: array_like
+    :param color: color of the lines defining the frame
+    :type color: str
+    :param textcolor: color of text labels for the frame, default color of lines above
+    :type textcolor: str
+    :param frame: label the frame, name is shown below the frame and as subscripts on the frame axis labels
+    :type frame: str
+    :param labels: labels for the axes, defaults to X, Y and Z
+    :type labels: 3-tuple of strings
+    :param length: length of coordinate frame axes, default 1
+    :type length: float
+    :param arrow: show arrow heads, default True
+    :type arrow: bool
+    :param wtl: width-to-length ratio for arrows, default 0.2
+    :type wtl: float
+    :param rviz: show Rviz style arrows, default False
+    :type rviz: bool
+    :param projection: 3D projection: ortho [default] or persp
+    :type projection: str
+    :param width: width of lines, default 1
+    :type width: float
+    :param d1: distance of frame axis label text from origin, default 1.15
+    :type d2: distance of frame label text from origin, default 0.05
+    :return: axes containing the frame
+    :rtype: Axes3DSubplot
 
-        - If no current figure, one is created
-        - If current figure, but no axes, a 3d Axes is created
+    Adds a 3D coordinate frame represented by the SO(3) or SE(3) matrix to the current axes.
 
-        Examples:
+    - If no current figure, one is created
+    - If current figure, but no axes, a 3d Axes is created
 
-             trplot(T, frame='A')
-             trplot(T, frame='A', color='green')
-             trplot(T1, 'labels', 'NOA');
+    Examples:
 
-        """
+            trplot(T, frame='A')
+            trplot(T, frame='A', color='green')
+            trplot(T1, 'labels', 'NOA');
 
-        # TODO
-        # animation
-        # anaglyph
+    """
 
-        # check input types
-        if isrot(T, check=True):
-            T = trn.r2t(T)
+    # TODO
+    # animation
+    # anaglyph
+
+    if not _matplotlib_exists:
+        print('matplotlib is not installed: pip install matplotlib')
+        return
+
+    # check input types
+    if isrot(T, check=True):
+        T = trn.r2t(T)
+    else:
+        assert ishom(T, check=True)
+
+    if axes is None:
+        # create an axes
+        fig = plt.gcf()
+        if fig.axes == []:
+            # no axes in the figure, create a 3D axes
+            ax = fig.add_subplot(111, projection='3d', proj_type=projection)
+            ax.autoscale(enable=True, axis='both')
+
+            # ax.set_aspect('equal')
+            ax.set_xlabel(labels[0])
+            ax.set_ylabel(labels[1])
+            ax.set_zlabel(labels[2])
         else:
-            assert ishom(T, check=True)
+            # reuse an existing axis
+            ax = plt.gca()
+    else:
+        ax = axes
 
-        if axes is None:
-            # create an axes
-            fig = plt.gcf()
-            if fig.axes == []:
-                # no axes in the figure, create a 3D axes
-                ax = fig.add_subplot(111, projection='3d', proj_type=projection)
-                ax.autoscale(enable=True, axis='both')
+    if dims is not None:
+        if len(dims) == 2:
+            dims = dims * 3
+        ax.set_xlim(dims[0:2])
+        ax.set_ylim(dims[2:4])
+        ax.set_zlim(dims[4:6])
 
-                # ax.set_aspect('equal')
-                ax.set_xlabel(labels[0])
-                ax.set_ylabel(labels[1])
-                ax.set_zlabel(labels[2])
-            else:
-                # reuse an existing axis
-                ax = plt.gca()
-        else:
-            ax = axes
+    # create unit vectors in homogeneous form
+    o = T @ np.array([0, 0, 0, 1])
+    x = T @ np.array([1, 0, 0, 1]) * length
+    y = T @ np.array([0, 1, 0, 1]) * length
+    z = T @ np.array([0, 0, 1, 1]) * length
 
-        if dims is not None:
-            if len(dims) == 2:
-                dims = dims * 3
-            ax.set_xlim(dims[0:2])
-            ax.set_ylim(dims[2:4])
-            ax.set_zlim(dims[4:6])
+    # draw the axes
 
-        # create unit vectors in homogeneous form
-        o = T @ np.array([0, 0, 0, 1])
-        x = T @ np.array([1, 0, 0, 1]) * length
-        y = T @ np.array([0, 1, 0, 1]) * length
-        z = T @ np.array([0, 0, 1, 1]) * length
+    if rviz:
+        ax.plot([o[0], x[0]], [o[1], x[1]], [o[2], x[2]], color='red', linewidth=5 * width)
+        ax.plot([o[0], y[0]], [o[1], y[1]], [o[2], y[2]], color='lime', linewidth=5 * width)
+        ax.plot([o[0], z[0]], [o[1], z[1]], [o[2], z[2]], color='blue', linewidth=5 * width)
+    elif arrow:
+        ax.quiver(o[0], o[1], o[2], x[0] - o[0], x[1] - o[1], x[2] - o[2], arrow_length_ratio=wtl, linewidth=width, facecolor=color, edgecolor=color)
+        ax.quiver(o[0], o[1], o[2], y[0] - o[0], y[1] - o[1], y[2] - o[2], arrow_length_ratio=wtl, linewidth=width, facecolor=color, edgecolor=color)
+        ax.quiver(o[0], o[1], o[2], z[0] - o[0], z[1] - o[1], z[2] - o[2], arrow_length_ratio=wtl, linewidth=width, facecolor=color, edgecolor=color)
+        # plot an invisible point at the end of each arrow to allow auto-scaling to work
+        ax.scatter(xs=[o[0], x[0], y[0], z[0]], ys=[o[1], x[1], y[1], z[1]], zs=[o[2], x[2], y[2], z[2]], s=[20, 0, 0, 0])
+    else:
+        ax.plot([o[0], x[0]], [o[1], x[1]], [o[2], x[2]], color=color, linewidth=width)
+        ax.plot([o[0], y[0]], [o[1], y[1]], [o[2], y[2]], color=color, linewidth=width)
+        ax.plot([o[0], z[0]], [o[1], z[1]], [o[2], z[2]], color=color, linewidth=width)
 
-        # draw the axes
+    # label the frame
+    if frame:
+        if textcolor is not None:
+            color = textcolor
 
-        if rviz:
-            ax.plot([o[0], x[0]], [o[1], x[1]], [o[2], x[2]], color='red', linewidth=5 * width)
-            ax.plot([o[0], y[0]], [o[1], y[1]], [o[2], y[2]], color='lime', linewidth=5 * width)
-            ax.plot([o[0], z[0]], [o[1], z[1]], [o[2], z[2]], color='blue', linewidth=5 * width)
-        elif arrow:
-            ax.quiver(o[0], o[1], o[2], x[0] - o[0], x[1] - o[1], x[2] - o[2], arrow_length_ratio=wtl, linewidth=width, facecolor=color, edgecolor=color)
-            ax.quiver(o[0], o[1], o[2], y[0] - o[0], y[1] - o[1], y[2] - o[2], arrow_length_ratio=wtl, linewidth=width, facecolor=color, edgecolor=color)
-            ax.quiver(o[0], o[1], o[2], z[0] - o[0], z[1] - o[1], z[2] - o[2], arrow_length_ratio=wtl, linewidth=width, facecolor=color, edgecolor=color)
-            # plot an invisible point at the end of each arrow to allow auto-scaling to work
-            ax.scatter(xs=[o[0], x[0], y[0], z[0]], ys=[o[1], x[1], y[1], z[1]], zs=[o[2], x[2], y[2], z[2]], s=[20, 0, 0, 0])
-        else:
-            ax.plot([o[0], x[0]], [o[1], x[1]], [o[2], x[2]], color=color, linewidth=width)
-            ax.plot([o[0], y[0]], [o[1], y[1]], [o[2], y[2]], color=color, linewidth=width)
-            ax.plot([o[0], z[0]], [o[1], z[1]], [o[2], z[2]], color=color, linewidth=width)
+        o1 = T @ np.array([-d1, -d1, -d1, 1])
+        ax.text(o1[0], o1[1], o1[2], r'$\{' + frame + r'\}$', color=color, verticalalignment='top', horizontalalignment='center')
 
-        # label the frame
-        if frame:
-            if textcolor is not None:
-                color = textcolor
+        # add the labels to each axis
 
-            o1 = T @ np.array([-d1, -d1, -d1, 1])
-            ax.text(o1[0], o1[1], o1[2], r'$\{' + frame + r'\}$', color=color, verticalalignment='top', horizontalalignment='center')
+        x = (x - o) * d2 + o
+        y = (y - o) * d2 + o
+        z = (z - o) * d2 + o
 
-            # add the labels to each axis
+        ax.text(x[0], x[1], x[2], "$%c_{%s}$" % (labels[0], frame), color=color, horizontalalignment='center', verticalalignment='center')
+        ax.text(y[0], y[1], y[2], "$%c_{%s}$" % (labels[1], frame), color=color, horizontalalignment='center', verticalalignment='center')
+        ax.text(z[0], z[1], z[2], "$%c_{%s}$" % (labels[2], frame), color=color, horizontalalignment='center', verticalalignment='center')
 
-            x = (x - o) * d2 + o
-            y = (y - o) * d2 + o
-            z = (z - o) * d2 + o
+    if block:
+        # calling this at all, causes FuncAnimation to fail so when invoked from tranimate skip this bit
+        plt.show(block=block)
+    return ax
 
-            ax.text(x[0], x[1], x[2], "$%c_{%s}$" % (labels[0], frame), color=color, horizontalalignment='center', verticalalignment='center')
-            ax.text(y[0], y[1], y[2], "$%c_{%s}$" % (labels[1], frame), color=color, horizontalalignment='center', verticalalignment='center')
-            ax.text(z[0], z[1], z[2], "$%c_{%s}$" % (labels[2], frame), color=color, horizontalalignment='center', verticalalignment='center')
-        
-        if block:
-            # calling this at all, causes FuncAnimation to fail so when invoked from tranimate skip this bit
-            plt.show(block=block)
-        return ax
+def tranimate(T, **kwargs):
+    """
+    Animate a 3D coordinate frame
 
-    from spatialmath.base import animate as animate
+    :param T: an SO(3) or SE(3) pose to be displayed as coordinate frame
+    :type: numpy.ndarray, shape=(3,3) or (4,4)
+    :param nframes: number of steps in the animation [defaault 100]
+    :type nframes: int
+    :param repeat: animate in endless loop [default False]
+    :type repeat: bool
+    :param interval: number of milliseconds between frames [default 50]
+    :type interval: int
+    :param movie: name of file to write MP4 movie into
+    :type movie: str
 
-        
-    def tranimate(T, **kwargs):
-        """
-        Animate a 3D coordinate frame
-    
-        :param T: an SO(3) or SE(3) pose to be displayed as coordinate frame
-        :type: numpy.ndarray, shape=(3,3) or (4,4)
-        :param nframes: number of steps in the animation [defaault 100]
-        :type nframes: int
-        :param repeat: animate in endless loop [default False]
-        :type repeat: bool
-        :param interval: number of milliseconds between frames [default 50]
-        :type interval: int
-        :param movie: name of file to write MP4 movie into
-        :type movie: str
-        
-        Animates a 3D coordinate frame moving from the world frame to a frame represented by the SO(3) or SE(3) matrix to the current axes.
-    
-        - If no current figure, one is created
-        - If current figure, but no axes, a 3d Axes is created
-        
-    
-        Examples:
-    
-             tranimate(transl(1,2,3)@trotx(1), frame='A', arrow=False, dims=[0, 5])
-             tranimate(transl(1,2,3)@trotx(1), frame='A', arrow=False, dims=[0, 5], movie='spin.mp4')
-        """
-        anim = animate.Animate(**kwargs)
-        anim.trplot(T, **kwargs)
-        anim.run(**kwargs)
+    Animates a 3D coordinate frame moving from the world frame to a frame represented by the SO(3) or SE(3) matrix to the current axes.
+
+    - If no current figure, one is created
+    - If current figure, but no axes, a 3d Axes is created
+
+
+    Examples:
+
+            tranimate(transl(1,2,3)@trotx(1), frame='A', arrow=False, dims=[0, 5])
+            tranimate(transl(1,2,3)@trotx(1), frame='A', arrow=False, dims=[0, 5], movie='spin.mp4')
+    """
+    if not _matplotlib_exists:
+        print('matplotlib is not installed: pip install matplotlib')
+        return
+
+    anim = animate.Animate(**kwargs)
+    anim.trplot(T, **kwargs)
+    anim.run(**kwargs)
 
 if __name__ == '__main__':  # pragma: no cover
     import pathlib
-    import os.path
 
-    exec(open(os.path.join(pathlib.Path(__file__).parent.absolute(), "test_transforms.py")).read())
+    exec(open(pathlib.Path(__file__).parent.absolute() / "test_transforms.py").read())  # pylint: disable=exec-used
